using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using Benday.MvvmTips.ViewModels;
using Microsoft.Silverlight.Testing;

namespace Benday.MvvmTips.UnitTests
{
    [TestClass]
    [Tag("Collections")]
    public class CollectionsAndVisibilitySampleViewModelFixture : SilverlightUnitTestBase
    {
        [TestMethod]
        public void InitialValuesAreCorrect()
        {
            var instance = new CollectionsAndVisibilitySampleViewModel();

            Assert.IsNotNull(instance.Fruits);
            Assert.IsNotNull(instance.CurrentFruit);
            Assert.IsNotNull(instance.ShowNextField);
            Assert.IsNotNull(instance.SecretMessage);

            Assert.AreNotEqual<int>(0, instance.Fruits.Items.Count, "Nothing in the fruits collection.");
            Assert.AreEqual<string>(String.Empty, instance.CurrentFruit.Value, "Current fruit should be empty.");
            Assert.IsFalse(instance.ShowNextField.Value, "Show next field should be false.");
            Assert.IsFalse(instance.SecretMessage.IsVisible, "Secret message field should not be visible.");

            Assert.AreEqual<string>("Quit being so paranoid.", instance.SecretMessage.Value, "Secret message value was wrong.");
        }

        [TestMethod]
        public void ChangingSelectedFruitUpdatesCurrentFruitValue()
        {
            var instance = new CollectionsAndVisibilitySampleViewModel();

            instance.Fruits.SelectByText("Apple");
            Assert.AreEqual<string>("Apple", instance.CurrentFruit.Value, "Current fruit value was wrong.");

            instance.Fruits.SelectByText("Lychee");
            Assert.AreEqual<string>("Lychee", instance.CurrentFruit.Value, "Current fruit value was wrong.");
        }

        [TestMethod]
        public void ChangingShowNextFieldUpdatesVisibilityForSecretMessage()
        {
            var instance = new CollectionsAndVisibilitySampleViewModel();

            instance.ShowNextField.Value = true;
            Assert.IsTrue(instance.SecretMessage.IsVisible, "Secret message field visibility was wrong.");

            instance.ShowNextField.Value = false;
            Assert.IsFalse(instance.SecretMessage.IsVisible, "Secret message field visibility was wrong.");
        }








    }
}
