using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.MvvmTips.ViewModels;
using Benday.MvvmTips.Models;
using Benday.MvvmTips.WcfDataAccess;

namespace Benday.MvvmTips.UnitTests
{
    [TestClass]
    public class PersonModelToServiceDtoAdapterFixture : SilverlightUnitTestBase
    {
        protected override void OnTestInitialize()
        {
            m_AdapterInstance = null;
            m_ServiceDtoInstance = null;
            m_ModelInstance = null;
        }

        [TestMethod]
        public void AdaptModelToServiceDto()
        {
            UnitTestUtility.PopulateWithTestData(ModelInstance);

            AdapterInstance.Adapt(ModelInstance, ServiceDtoInstance);

            AssertAreEqual(ModelInstance, ServiceDtoInstance);
        }

        [TestMethod]
        public void AdaptServiceDtoToModel()
        {
            PopulateWithTestData(ServiceDtoInstance);

            AdapterInstance.Adapt(ServiceDtoInstance, ModelInstance);

            AssertAreEqual(ServiceDtoInstance, ModelInstance);
        }

        private void PopulateWithTestData(PersonService.PersonDto populateThis)
        {
            populateThis.Id = UnitTestUtility.GetReasonablyUniqueInt("Id");
            populateThis.EmailAddress = UnitTestUtility.GetReasonablyUniqueString("Email");
            populateThis.FirstName = UnitTestUtility.GetReasonablyUniqueString("FirstName");
            populateThis.LastName = UnitTestUtility.GetReasonablyUniqueString("LastName");
        }

        private void AssertAreEqual(IPerson expected, PersonService.PersonDto actual)
        {
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
        }

        private void AssertAreEqual(PersonService.PersonDto expected, IPerson actual)
        {
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
        }

        private PersonModelToServiceDtoAdapter m_AdapterInstance;
        public PersonModelToServiceDtoAdapter AdapterInstance
        {
            get
            {
                if (m_AdapterInstance == null)
                {
                    m_AdapterInstance = new PersonModelToServiceDtoAdapter();
                }

                return m_AdapterInstance;
            }
        }

        private PersonService.PersonDto m_ServiceDtoInstance;
        public PersonService.PersonDto ServiceDtoInstance
        {
            get
            {
                if (m_ServiceDtoInstance == null)
                {
                    m_ServiceDtoInstance = new PersonService.PersonDto();
                }

                return m_ServiceDtoInstance;
            }
        }

        private IPerson m_ModelInstance;
        public IPerson ModelInstance
        {
            get
            {
                if (m_ModelInstance == null)
                {
                    m_ModelInstance = new Person();
                }

                return m_ModelInstance;
            }
        }
    }
}
