using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.MvvmTips.ViewModels;
using Benday.MvvmTips.Models;
using Benday.MvvmTips.WcfDataAccess;

namespace Benday.MvvmTips.UnitTests
{
    public class PersonRepositoryMock : IPersonRepository
    {
        public void Reset()
        {
            WasLoadByIdCalled = false;
            ModelToReturnFromLoadByIdMethod = null;
            IdReceivedByLoadByIdMethod = Int32.MinValue;

            WasSaveCalled = false;
            ModelReceivedBySaveMethod = null;
        }

        public void Reset(IPerson modelToReturnFromLoadById)
        {
            Reset();

            ModelToReturnFromLoadByIdMethod = modelToReturnFromLoadById;
        }

        public bool WasLoadByIdCalled { get; set; }
        public IPerson ModelToReturnFromLoadByIdMethod { get; set; }
        public int IdReceivedByLoadByIdMethod { get; set; }

        public bool WasSaveCalled { get; set; }
        public IPerson ModelReceivedBySaveMethod { get; set; }

        public void LoadById(Common.ReturnResult<IPerson> callback, int id)
        {
            WasLoadByIdCalled = true;
            IdReceivedByLoadByIdMethod = id;
            callback.Notify(ModelToReturnFromLoadByIdMethod);
        }

        public void Save(Common.ReturnResult<IPerson> callback, IPerson saveThis)
        {
            WasSaveCalled = true;
            ModelReceivedBySaveMethod = saveThis;
            if (saveThis != null && saveThis.Id == 0)
            {
                // pretend the database returned a new @@IDENTITY value
                saveThis.Id = UnitTestUtility.GetReasonablyUniqueInt("Id");
            }

            callback.Notify(saveThis);
        }
    }
}
