﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.MvvmTips.Factories;
using Benday.MvvmTips.Common;
using Rhino.Mocks;

namespace Benday.MvvmTips.UnitTests
{
    public abstract class SilverlightUnitTestBase : SilverlightTest
    {
        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }

        [ClassInitialize]
        public void ClassInitialize()
        {
            OnClassInitialize();
        }

        private ObjectFactoryConfiguration m_additionalObjectFactoryConfiguration;
        public ObjectFactoryConfiguration AdditionalObjectFactoryConfiguration
        {
            get
            {
                if (m_additionalObjectFactoryConfiguration == null)
                {
                    m_additionalObjectFactoryConfiguration = new ObjectFactoryConfiguration();

                    PopulateAdditionalObjectFactoryConfiguration(m_additionalObjectFactoryConfiguration);
                }

                return m_additionalObjectFactoryConfiguration;
            }
        }

        protected virtual void PopulateAdditionalObjectFactoryConfiguration(ObjectFactoryConfiguration configuration)
        {

        }

        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory(AdditionalObjectFactoryConfiguration);
                }

                return m_objectFactoryInstance;
            }
        }

        public void ResetObjectFactoryInstance()
        {
            m_objectFactoryInstance = null;
        }

        protected MockRepository m_mockRepositoryInstance;
        public MockRepository MockRepositoryInstance
        {
            get
            {
                if (m_mockRepositoryInstance == null)
                {
                    m_mockRepositoryInstance = new MockRepository();
                }

                return m_mockRepositoryInstance;
            }
        }

        protected virtual void OnTestInitialize()
        {
            m_objectFactoryInstance = null;
            m_additionalObjectFactoryConfiguration = null;
            m_mockRepositoryInstance = null;
        }

        protected virtual void OnClassInitialize()
        {
        }
    }
}