using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.MvvmTips.ViewModels;
using Benday.MvvmTips.Models;

namespace Benday.MvvmTips.UnitTests
{
    public static class UnitTestUtility
    {
        public static void PopulateWithTestData(IPerson populateThis)
        {
            populateThis.Id = GetReasonablyUniqueInt("Id");
            populateThis.EmailAddress = GetReasonablyUniqueString("Email");
            populateThis.FirstName = GetReasonablyUniqueString("FirstName");
            populateThis.LastName = GetReasonablyUniqueString("LastName");
        }

        public static void PopulateWithTestData(IPersonViewModel populateThis)
        {
            populateThis.Id.Value = GetReasonablyUniqueInt("Id");
            populateThis.EmailAddress.Value = GetReasonablyUniqueString("Email");
            populateThis.FirstName.Value = GetReasonablyUniqueString("FirstName");
            populateThis.LastName.Value = GetReasonablyUniqueString("LastName");
        }

        public static void AssertAreEqual(IPerson expected, IPersonViewModel actual)
        {
            Assert.AreEqual<int>(expected.Id, actual.Id.Value, "Id");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress.Value, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName.Value, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName.Value, "LastName");
        }

        public static void AssertAreNotEqual(IPerson expected, IPersonViewModel actual)
        {
            Assert.AreNotEqual<int>(expected.Id, actual.Id.Value, "Id");
            Assert.AreNotEqual<string>(expected.EmailAddress, actual.EmailAddress.Value, "EmailAddress");
            Assert.AreNotEqual<string>(expected.FirstName, actual.FirstName.Value, "FirstName");
            Assert.AreNotEqual<string>(expected.LastName, actual.LastName.Value, "LastName");
        }

        public static void AssertAreEqual(IPersonViewModel expected, IPerson actual)
        {
            Assert.AreEqual<int>(expected.Id.Value, actual.Id, "Id");
            Assert.AreEqual<string>(expected.EmailAddress.Value, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName.Value, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName.Value, actual.LastName, "LastName");
        }

        public static int GetReasonablyUniqueInt(string prefix)
        {
            var guidAsByteArray = Guid.NewGuid().ToByteArray();

            int total = 0;

            foreach (byte byteFromGuid in guidAsByteArray)
            {
                total += ((int)byteFromGuid);
            }

            return prefix.Length + total;
        }

        public static string GetReasonablyUniqueString(string prefix)
        {
            string fullValue = String.Format("{0}{1}", prefix, Guid.NewGuid());

            if (fullValue.Length > 20)
            {
                return fullValue.Substring(0, 20);
            }
            else
            {
                return fullValue;
            }
        }

        public static void AssertFieldValue<T>(T expected, ViewModelField<T> fieldInstance, string fieldName)
        {
            Assert.IsNotNull(fieldInstance, "Field instance for '{0}' was null.", fieldName);

            var actual = fieldInstance.Value;

            Assert.AreEqual<T>(expected, actual, "Field value for '{0}' did not match.", fieldName);
        }
    }
}
