using System;
using GalaSoft.MvvmLight;
using Benday.MvvmTips.Common;
using System.Collections.Generic;

namespace Benday.MvvmTips.ViewModels
{
    public class CollectionsAndVisibilitySampleViewModel : ViewModelBase
    {
        public CollectionsAndVisibilitySampleViewModel()
        {
            InitializeFieldInstances();
        }

        private void InitializeFieldInstances()
        {
            CurrentFruit = new ViewModelField<string>(String.Empty);
            SecretMessage = new ViewModelField<string>("Quit being so paranoid.");
            SecretMessage.IsVisible = false;

            ShowNextField = new ViewModelField<bool>(false);
            ShowNextField.OnValueChanged += new EventHandler(ShowNextField_OnValueChanged);            
            
            Fruits = new SingleSelectListViewModel(GetListOfFruit());
            Fruits.OnItemSelected += new EventHandler(Fruits_OnItemSelected);
        }

        void ShowNextField_OnValueChanged(object sender, EventArgs e)
        {
            SecretMessage.IsVisible = ShowNextField.Value;
        }

        void Fruits_OnItemSelected(object sender, EventArgs e)
        {
            var selectedItem = Fruits.SelectedItem;

            if (selectedItem != null)
            {
                CurrentFruit.Value = selectedItem.Text;
            }
            else
            {
                CurrentFruit.Value = "(no fruit selected)";
            }
        }

        private IEnumerable<ISelectableItem> GetListOfFruit()
        {
            var items = new List<ISelectableItem>();

            items.Add(new SelectableItem(false, "Apple", "Apple"));
            items.Add(new SelectableItem(false, "Banana", "Banana"));
            items.Add(new SelectableItem(false, "Cherry", "Cherry"));
            items.Add(new SelectableItem(false, "Grape", "Grape"));
            items.Add(new SelectableItem(false, "Lychee", "Lychee"));

            return items;
        }

        private const string FruitsPropertyName = "Fruits";

        private SingleSelectListViewModel m_Fruits;
        public SingleSelectListViewModel Fruits
        {
            get
            {
                return m_Fruits;
            }
            set
            {
                m_Fruits = value;
                RaisePropertyChanged(FruitsPropertyName);
            }
        }

        private const string CurrentFruitPropertyName = "CurrentFruit";

        private ViewModelField<string> m_CurrentFruit;
        public ViewModelField<string> CurrentFruit
        {
            get
            {
                return m_CurrentFruit;
            }
            set
            {
                m_CurrentFruit = value;
                RaisePropertyChanged(CurrentFruitPropertyName);
            }
        }

        private const string ShowNextFieldPropertyName = "ShowNextField";

        private ViewModelField<bool> m_ShowNextField;
        public ViewModelField<bool> ShowNextField
        {
            get
            {
                return m_ShowNextField;
            }
            set
            {
                m_ShowNextField = value;
                RaisePropertyChanged(ShowNextFieldPropertyName);
            }
        }

        private const string SecretMessagePropertyName = "SecretMessage";

        private ViewModelField<string> m_SecretMessage;
        public ViewModelField<string> SecretMessage
        {
            get
            {
                return m_SecretMessage;
            }
            set
            {
                m_SecretMessage = value;
                RaisePropertyChanged(SecretMessagePropertyName);
            }
        }
    }
}
