﻿using System;
using System.Windows;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using Benday.MvvmTips.Common;
using System.Threading;
using GalaSoft.MvvmLight.Command;

namespace Benday.MvvmTips.ViewModels
{
    public class ProgressBarSampleViewModel : ViewModelBase, IProgressBarSampleViewModel
    {
        private const int DELAY_IN_MILLISECS = 1000;

        public ProgressBarSampleViewModel() 
        {
            RunProgressBarCommand = new RelayCommand(RunProgressBar);
        }

        public ICommand RunProgressBarCommand { get; set; }

        public void RunProgressBar()
        {
            IsProgressBarVisible = true;
            ProgressBarMessage = "Initializing...";
            ThreadPool.QueueUserWorkItem(new WaitCallback(RunProgressBar));
        }

        private static void WaitBriefly()
        {
            Thread.Sleep(DELAY_IN_MILLISECS);
        }

        private void SetMessage(string message)
        {
            Deployment.Current.Dispatcher.BeginInvoke(
                () => ProgressBarMessage = message);
        }

        private void RunProgressBar(object state)
        {
            SetMessage("Starting to do stuff...");
            WaitBriefly();

            SetMessage("Doing stuff...");
            WaitBriefly();

            SetMessage("Watching TV...");
            WaitBriefly();

            SetMessage("Getting some coffee...");
            WaitBriefly();

            SetMessage("Finished.");
            WaitBriefly();

            Deployment.Current.Dispatcher.BeginInvoke(
                () => IsProgressBarVisible = false);            
        }

        public event MessageBoxEventHandler MessageBoxRequested;

        private const string IsProgressBarVisiblePropertyName = "IsProgressBarVisible";

        private bool m_IsProgressBarVisible;
        public bool IsProgressBarVisible
        {
            get
            {
                return m_IsProgressBarVisible;
            }
            set
            {
                m_IsProgressBarVisible = value;
                RaisePropertyChanged(IsProgressBarVisiblePropertyName);
            }
        }

        private const string ProgressBarMessagePropertyName = "ProgressBarMessage";

        private string m_ProgressBarMessage;
        public string ProgressBarMessage
        {
            get
            {
                return m_ProgressBarMessage;
            }
            set
            {
                m_ProgressBarMessage = value;
                RaisePropertyChanged(ProgressBarMessagePropertyName);
            }
        }
    }
}
