using System;
using GalaSoft.MvvmLight;
using Microsoft.Practices.Unity;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Benday.MvvmTips.Common;

namespace Benday.MvvmTips.ViewModels
{
    public class SelectableItem : ViewModelBase, ISelectableItem
    {
        [InjectionConstructor]
        public SelectableItem()
        {

        }

        /// <summary>
        /// Initializes a new instance of the SelectableItem class.
        /// </summary>
        /// <param name="isSelected"></param>
        /// <param name="text"></param>
        public SelectableItem(bool isSelected, string text)
            : this(isSelected, text, text)
        {

        }

        /// <summary>
        /// Initializes a new instance of the SelectableItemViewModel class.
        /// </summary>
        /// <param name="isSelected"></param>
        /// <param name="text"></param>
        /// <param name="value"></param>
        public SelectableItem(bool isSelected, string text, string value)
        {
            m_isSelected = isSelected;
            m_text = text;
            m_value = value;
        }

        private const string IsSelectedPropertyName = "IsSelected";

        private bool m_isSelected;
        public bool IsSelected
        {
            get
            {
                return m_isSelected;
            }
            set
            {
                if (m_isSelected != value)
                {
                    m_isSelected = value;
                    RaisePropertyChanged(IsSelectedPropertyName);    
                }                
            }
        }

        private const string TextPropertyName = "Text";

        private string m_text;
        public string Text
        {
            get
            {
                return m_text;
            }
            set
            {
                m_text = value;
                RaisePropertyChanged(TextPropertyName);
            }
        }

        private const string ValuePropertyName = "Value";

        private string m_value;
        public string Value
        {
            get
            {
                return m_value;
            }
            set
            {
                m_value = value;
                RaisePropertyChanged(ValuePropertyName);
            }
        }

        public override string ToString()
        {
            return Text;
        }

        private const string DescriptionPropertyName = "Description";

        private string m_Description;
        public string Description
        {
            get
            {
                return m_Description;
            }
            set
            {
                m_Description = value;
                RaisePropertyChanged(DescriptionPropertyName);
            }
        }

        private const string IdPropertyName = "Id";

        private int m_Id;
        public int Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }
    }
}
