using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using System.ComponentModel;
using System.Collections.Generic;

namespace Benday.MvvmTips.ViewModels
{
    public class ViewModelField<T> : ViewModelBase, INotifyPropertyChanged
    {
        public ViewModelField()
        {
            IsValid = true;
            IsVisible = true;
            ValidationMessage = String.Empty;
        }

        public ViewModelField(T initialValue) : this()
        {
            m_Value = initialValue;
        }

        private T m_Value;
        public T Value
        {
            get { return m_Value; }
            set
            {
                if (EqualityComparer<T>.Default.Equals(
                    m_Value, value) == false)
                {
                    m_Value = value;
                    RaisePropertyChanged("Value");
                    RaiseOnValueChanged();
                }
            }
        }
        
        public event EventHandler OnValueChanged;
        public virtual void RaiseOnValueChanged()
        {
            EventHandler handler = OnValueChanged;

            if (handler != null)
            {
                handler(this, new EventArgs());
            }
        }

        private const string IsVisiblePropertyName = "IsVisible";

        private bool m_IsVisible;
        public bool IsVisible
        {
            get
            {
                return m_IsVisible;
            }
            set
            {
                m_IsVisible = value;
                RaisePropertyChanged(IsVisiblePropertyName);
            }
        }

        private const string IsValidPropertyName = "IsValid";

        private bool m_IsValid;
        public bool IsValid
        {
            get
            {
                return m_IsValid;
            }
            set
            {
                m_IsValid = value;
                RaisePropertyChanged(IsValidPropertyName);
            }
        }

        private const string ValidationMessagePropertyName = "ValidationMessage";

        private string m_ValidationMessage;
        public string ValidationMessage
        {
            get
            {
                return m_ValidationMessage;
            }
            set
            {
                m_ValidationMessage = value;
                RaisePropertyChanged(ValidationMessagePropertyName);
            }
        }

        public override string ToString()
        {
            if (Value == null)
            {
                return String.Empty;
            }
            else
            {
                return Value.ToString();
            }
        }
    }
}
