using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Benday.MvvmTips.Common;
using Benday.MvvmTips.Models;
using Benday.MvvmTips.PersonService;

namespace Benday.MvvmTips.WcfDataAccess
{
    public class AsyncWcfPersonRepository : IPersonRepository
    {
        public void LoadById(ReturnResult<IPerson> callback, int id)
        {
            // create an instance of the service proxy
            var client = new PersonService.PersonServiceClient();

            // subscribe to the "completed" event for the service method
            client.LoadByIdCompleted +=
                new EventHandler<PersonService.LoadByIdCompletedEventArgs>(
                    client_LoadByIdCompleted);

            // call the service method
            client.LoadByIdAsync(id, callback);
        }

        void client_LoadByIdCompleted(object sender,
            PersonService.LoadByIdCompletedEventArgs e)
        {
            var callback = e.UserState as ReturnResult<IPerson>;

            if (e.Error != null)
            {
                // pass the WCF exception to the original caller
                callback.Notify(e.Error);
            }
            else
            {
                PersonService.PersonDto personReturnedByService = e.Result;

                var returnValue = new Person();

                var adapter = new PersonModelToServiceDtoAdapter();

                adapter.Adapt(personReturnedByService, returnValue);

                // pass the populated model to the original caller
                callback.Notify(returnValue);    
            }            
        }

        public void Save(ReturnResult<IPerson> callback, IPerson saveThis)
        {
            // create an instance of the service proxy
            var client = new PersonService.PersonServiceClient();

            // subscribe to the "completed" event for the service method
            client.SaveCompleted +=
                new EventHandler<PersonService.SaveCompletedEventArgs>(
                    client_SaveCompleted);

            var adapter = new PersonModelToServiceDtoAdapter();

            var saveThisAsServiceProxyDtoObject = new PersonDto();

            adapter.Adapt(saveThis, 
                saveThisAsServiceProxyDtoObject);

            // call the service method
            client.SaveAsync(saveThisAsServiceProxyDtoObject, callback);
        }

        void client_SaveCompleted(object sender,
            PersonService.SaveCompletedEventArgs e)
        {
            var callback = e.UserState as ReturnResult<IPerson>;

            if (e.Error != null)
            {
                // pass the WCF exception to the original caller
                callback.Notify(e.Error);
            }
            else
            {
                PersonService.PersonDto personReturnedByService = e.Result;

                IPerson returnValue = new Person();

                var adapter = new PersonModelToServiceDtoAdapter();

                adapter.Adapt(personReturnedByService, returnValue);

                // pass the populated model to the original caller
                callback.Notify(returnValue);
            }
        }
    }
}
